@@ -25,7 +25,6 @@ def send_warnings_to_stdout(message, category, filename, lineno,
     #TODO - Have Biopython DataLossWarning?
     if category in [UserWarning]:
         print "%s - %s" % (category.__name__, message)
-warnings.resetwarnings()
 warnings.showwarning = send_warnings_to_stdout
 
 
@@ -34,14 +33,16 @@ dna_alphas = [Alphabet.generic_dna]
 rna_alphas = [Alphabet.generic_rna]
 nucleotide_alphas = [Alphabet.generic_nucleotide,
                      Alphabet.Gapped(Alphabet.generic_nucleotide)]
-no_alpha_formats = ["fasta","clustal","phylip","tab","ig","stockholm","emboss",
-                    "fastq","fastq-solexa","fastq-illumina","qual"]
-possible_unknown_seq_formats = ["qual", "genbank", "gb", "embl"]
+no_alpha_formats = ["fasta","clustal","phylip","phylip-relaxed","tab","ig",
+                    "stockholm","emboss", "fastq","fastq-solexa",
+                    "fastq-illumina","qual"]
+possible_unknown_seq_formats = ["qual", "genbank", "gb", "embl", "imgt"]
 
 #List of formats including alignment only file formats we can read AND write.
 #The list is initially hard coded to preserve the original order of the unit
 #test output, with any new formats added since appended to the end.
-test_write_read_alignment_formats = ["fasta","clustal","phylip","stockholm"]
+test_write_read_alignment_formats = ["fasta","clustal","phylip","stockholm",
+                                     "phylip-relaxed"]
 for format in sorted(SeqIO._FormatToWriter):
     if format not in test_write_read_alignment_formats:
         test_write_read_alignment_formats.append(format)
@@ -115,6 +116,11 @@ test_files = [ \
     ("swiss",  False, 'SwissProt/sp016', 1),
 #Following example is also used in test_registry.py
     ("swiss",  False, 'Registry/EDD_RAT.dat', 1),
+#Following examples are also used in test_Uniprot.py
+    ("uniprot-xml",  False, 'SwissProt/uni001', 1),
+    ("uniprot-xml",  False, 'SwissProt/uni002', 3),
+    ("uniprot-xml",  False, 'SwissProt/Q13639.xml', 1),
+    ("swiss",    False, 'SwissProt/Q13639.txt', 1),
 #Following examples are also used in test_GenBank.py
     ("genbank",False, 'GenBank/noref.gb', 1),
     ("genbank",False, 'GenBank/cor6_6.gb', 6),
@@ -138,7 +144,8 @@ test_files = [ \
     ("genbank",False, 'GenBank/gbvrl1_start.seq', 3),
 #Following files are also used in test_GFF.py
     ("genbank",False, 'GFF/NC_001422.gbk', 1),
-#Following files are currently only used here:
+#Following files are currently only used here or in test_SeqIO_index.py:
+    ("embl",   False, 'EMBL/epo_prt_selection.embl', 9), #proteins
     ("embl",   False, 'EMBL/TRBG361.embl', 1),
     ("embl",   False, 'EMBL/DD231055_edited.embl', 1),
     ("embl",   False, 'EMBL/SC10H5.embl', 1), # Pre 2006 style ID line
@@ -147,6 +154,7 @@ test_files = [ \
     ("embl",   False, 'EMBL/AE017046.embl', 1), #See also NC_005816.gb
     ("embl",   False, 'EMBL/Human_contigs.embl', 2), #contigs, no sequences
     ("embl",   False, 'EMBL/A04195.imgt', 1), # features over indented for EMBL
+    ("imgt",   False, 'EMBL/A04195.imgt', 1), # features over indented for EMBL
     ("stockholm", True,  'Stockholm/simple.sth', 2),
     ("stockholm", True,  'Stockholm/funny.sth', 5),
 #Following PHYLIP files are currently only used here and in test_AlignIO.py,
@@ -194,46 +202,35 @@ test_files = [ \
     ("fastq-illumina", False,'Quality/illumina_faked.fastq', 1),
     ("fastq-solexa", False, 'Quality/solexa_faked.fastq', 1),
     ("fastq-solexa", True, 'Quality/solexa_example.fastq', 5),
+#Following examples are also used in test_SeqXML.py
+    ("seqxml", False, 'SeqXML/dna_example.xml', 4),
+    ("seqxml", False, 'SeqXML/rna_example.xml', 5),
+    ("seqxml", False, 'SeqXML/protein_example.xml', 5),
+#Following examples are also used in test_SeqIO_AbiIO.py
+    ("abi", False, 'Abi/310.ab1', 1),
+    ("abi", False, 'Abi/3100.ab1', 1),
+    ("abi", False, 'Abi/3730.ab1', 1),
     ]
 
-# This is a list of two-tuples.  Each tuple contains a
-# list of SeqRecord objects and a description (string)
-test_records = [
-    ([], "zero records"),
-    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",Alphabet.generic_protein), id="Alpha"),
-      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",Alphabet.generic_protein), id="Gamma"),
-      SeqRecord(Seq("DITHGVG",Alphabet.generic_protein), id="delta")],
-     "three peptides of different lengths"),
-    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",Alphabet.generic_protein), id="Alpha"),
-      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",Alphabet.generic_protein), id="Beta"),
-      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",Alphabet.generic_protein), id="Gamma")],
-     "three proteins alignment"),
-    ([SeqRecord(Seq("AATAAACCTTGCTGGCCATTGTGATCCATCCA",Alphabet.generic_dna), id="X"),
-      SeqRecord(Seq("ACTCAACCTTGCTGGTCATTGTGACCCCAGCA",Alphabet.generic_dna), id="Y"),
-      SeqRecord(Seq("TTTCCTCGGAGGCCAATCTGGATCAAGACCAT",Alphabet.generic_dna), id="Z")],
-     "three DNA sequence alignment"),
-    ([SeqRecord(Seq("AATAAACCTTGCTGGCCATTGTGATCCATCCA",Alphabet.generic_dna), id="X",
-                name="The\nMystery\rSequece:\r\nX"),
-      SeqRecord(Seq("ACTCAACCTTGCTGGTCATTGTGACCCCAGCA",Alphabet.generic_dna), id="Y",
-                description="an%sevil\rdescription right\nhere" % os.linesep),
-      SeqRecord(Seq("TTTCCTCGGAGGCCAATCTGGATCAAGACCAT",Alphabet.generic_dna), id="Z")],
-     "3 DNA seq alignment with CR/LF in name/descr"),
-    ([SeqRecord(Seq("CHSMAIKLSSEHNIPSGIANAL",Alphabet.generic_protein), id="Alpha"),
-      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",Alphabet.generic_protein), id="Beta"),
-      SeqRecord(Seq("VHGMAHPLGAFYNTPHGVANAI",Alphabet.generic_protein), id="Beta"),
-      SeqRecord(Seq("HNGFTALEGEIHHLTHGEKVAF",Alphabet.generic_protein), id="Gamma")],
-     "alignment with repeated record"),
-    ]
-# Meddle with the annotation too:
-assert test_records[4][1] == "3 DNA seq alignment with CR/LF in name/descr"
-# Add a list of strings,
-test_records[4][0][2].annotations["note"] = ["Note%salso" % os.linesep \
-                                    + "\r\nhas\n evil line\rbreaks!", "Wow"]
-# Add a simple string
-test_records[4][0][2].annotations["comment"] = "More%sof" % os.linesep \
-                                          + "\r\nthese\n evil line\rbreaks!"
-# Add a float too:
-test_records[4][0][2].annotations["weight"] = 2.5
+class ForwardOnlyHandle(object):
+    """Mimic a network handle without seek and tell methods etc."""
+    def __init__(self, handle):
+        self._handle = handle
+
+    def __iter__(self):
+        return iter(self._handle)
+
+    def read(self, length=None):
+        if length is None:
+            return self._handle.read()
+        else:
+            return self._handle.read(length)
+
+    def readline(self):
+        return self._handle.readline()
+
+    def close(self):
+        return self._handle.close()
 
 def compare_record(record_one, record_two):
     """This is meant to be a strict comparison for exact agreement..."""
@@ -314,13 +311,13 @@ def check_simple_write_read(records, indent=" "):
            #rather long, and it seems a bit pointless to record them.
            continue
         print indent+"Checking can write/read as '%s' format" % format
-        
+
         #Going to write to a handle...
         if format in SeqIO._BinaryFormats:
             handle = BytesIO()
         else:
             handle = StringIO()
-        
+
         try:
             c = SeqIO.write(sequences=records, handle=handle, format=format)
             assert c == len(records)
@@ -365,7 +362,7 @@ def check_simple_write_read(records, indent=" "):
             assert len(r1) == len(r2)
 
             #Check the sequence
-            if format in ["gb", "genbank", "embl"]:
+            if format in ["gb", "genbank", "embl", "imgt"]:
                 #The GenBank/EMBL parsers will convert to upper case.
                 if isinstance(r1.seq, UnknownSeq) \
                 and isinstance(r2.seq, UnknownSeq):
@@ -384,6 +381,9 @@ def check_simple_write_read(records, indent=" "):
             if format=="phylip":
                 assert r1.id.replace("[","").replace("]","")[:10] == r2.id, \
                        "'%s' vs '%s'" % (r1.id, r2.id)
+            elif format=="phylip-relaxed":
+                assert r1.id.replace(" ", "").replace(':', '|') == r2.id, \
+                        "'%s' vs '%s'" % (r1.id, r2.id)
             elif format=="clustal":
                 assert r1.id.replace(" ","_")[:30] == r2.id, \
                        "'%s' vs '%s'" % (r1.id, r2.id)
@@ -408,7 +408,7 @@ def check_simple_write_read(records, indent=" "):
 
 #Check parsers can cope with an empty file
 for t_format in SeqIO._FormatToIterator:
-    if t_format in SeqIO._BinaryFormats:
+    if t_format in SeqIO._BinaryFormats or t_format=="uniprot-xml":
         #Not allowed empty SFF files.
         continue
     handle = StringIO()
@@ -420,12 +420,14 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
         mode = "rb"
     else:
         mode = "r"
-    
+
     print "Testing reading %s format file %s" % (t_format, t_filename)
     assert os.path.isfile(t_filename), t_filename
 
     #Try as an iterator using handle
-    records  = list(SeqIO.parse(handle=open(t_filename,mode), format=t_format))
+    h = open(t_filename,mode)
+    records  = list(SeqIO.parse(handle=h, format=t_format))
+    h.close()
     assert len(records)  == t_count, \
          "Found %i records but expected %i" % (len(records), t_count)
 
@@ -437,22 +439,20 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
 
     #Try using the iterator with the next() method
     records3 = []
-    seq_iterator = SeqIO.parse(handle=open(t_filename,mode), format=t_format)
+    h = open(t_filename,mode)
+    seq_iterator = SeqIO.parse(handle=h, format=t_format)
     while True:
         try:
             record = seq_iterator.next()
         except StopIteration:
-            record = None
-        #Note that if the SeqRecord class has a __len__ method,
-        #and it has a zero-length sequence, this would fail an
-        #"if record" test.
-        if record is not None:
-            records3.append(record)
-        else:
             break
+        assert record is not None, "Should raise StopIteration not return None"
+        records3.append(record)
+    h.close()
 
     #Try a mixture of next() and list (a torture test!)
-    seq_iterator = SeqIO.parse(handle=open(t_filename,mode), format=t_format)
+    h = open(t_filename,mode)
+    seq_iterator = SeqIO.parse(handle=h, format=t_format)
     try:
         record = seq_iterator.next()
     except StopIteration:
@@ -463,9 +463,16 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
     else:
         records4 = []
     assert len(records4) == t_count
+    h.close()
 
     #Try a mixture of next() and for loop (a torture test!)
-    seq_iterator = SeqIO.parse(handle=open(t_filename,mode), format=t_format)
+    #with a forward-only-handle
+    if t_format == "abi":
+        #Temp hack
+        h = open(t_filename, mode)
+    else:
+        h = ForwardOnlyHandle(open(t_filename, mode))
+    seq_iterator = SeqIO.parse(h, format=t_format)
     try:
         record = seq_iterator.next()
     except StopIteration:
@@ -477,6 +484,7 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
     else:
         records5 = []
     assert len(records5) == t_count
+    h.close()
 
     for i in range(t_count):
         record = records[i]
@@ -506,8 +514,8 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
                 "Bad cross reference in dbxrefs: %s" % repr(ref)
         assert len(record.dbxrefs) == len(record.dbxrefs), \
                "Repeated cross reference in dbxrefs: %s" % repr(record.dbxrefs)
-                
-            
+
+
         #Check the lists obtained by the different methods agree
         assert compare_record(record, records2[i])
         assert compare_record(record, records3[i])
@@ -516,7 +524,7 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
 
         if i < 3:
             print record_summary(record)
-    # Only printed the only first three records: 0,1,2 
+    # Only printed the only first three records: 0,1,2
     if t_count > 4:
         print " ..."
     if t_count > 3:
@@ -524,7 +532,7 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
 
     # Check Bio.SeqIO.read(...)
     if t_count == 1:
-        record = SeqIO.read(handle=open(t_filename), format=t_format)
+        record = SeqIO.read(handle=open(t_filename,mode), format=t_format)
         assert isinstance(record, SeqRecord)
     else:
         try:
@@ -608,70 +616,3 @@ for (t_format, t_alignment, t_filename, t_count) in test_files:
     check_simple_write_read(records)
 
 print "Finished tested reading files"
-print
-print "Starting testing writing records"
-print "(Note that some of these are expected to 'fail' and say why)"
-print
-for (records, descr) in test_records:
-    print "Testing can write/read %s" % descr
-    for format in test_write_read_alignment_formats:
-        print " Checking can write/read as '%s' format" % format
-
-        #################
-        # Write records #
-        #################
-        if format in SeqIO._BinaryFormats:
-            handle = BytesIO()
-        else:
-            handle = StringIO()
-        try:
-            c = SeqIO.write(records, handle, format)
-            assert c == len(records)
-        except ValueError, e:
-            #This is expected to happen on many of the examples.
-            print " Failed: %s" % str(e)
-            continue #goto next test
-
-        #################
-        # Read records  #
-        #################
-        handle.seek(0)
-        try:
-            new_records = list(SeqIO.parse(handle, format))
-        except ValueError, e:
-            #THIS INDICATES A SIGNIFICANT PROBLEM,
-            #as we can't read the file we just wrote!
-            print " FAILED: %s" % str(e)
-            continue #goto next test
-
-        #################
-        # Check records #
-        #################
-        assert len(new_records) == len(records)
-        for record, new_record in zip(records, new_records):
-            if format == "nexus":
-                #The nexus parser will dis-ambiguate repeated record ids.
-                assert record.id == new_record.id or \
-                       new_record.id.startswith(record.id+".copy")
-            else:
-                assert record.id == new_record.id
-            assert record.seq.tostring() == new_record.seq.tostring()
-            #Using compare_record(record, new_record) is too strict
-
-        #Close now, after checking, so that it can be used at the console for debugging
-        handle.close()
-
-#Check writers can cope with no alignments
-for format in SeqIO._FormatToWriter:
-    if format in SeqIO._BinaryFormats:
-        handle = BytesIO()
-    else:
-        handle = StringIO()
-    try :
-        assert 0 == SeqIO.write([], handle, format), \
-               "Writing no records to %s format should work!" \
-               % t_format
-    except ValueError, err:
-        print "Writing no records to %s format failed: %s" % (format, err)
-
-print "Finished tested writing files"
